use linkme::distributed_slice;
use std::sync::Once;

#[distributed_slice]
static CTORS: [fn()] = [..];

#[distributed_slice]
static DTORS: [fn()] = [..];

static INIT: Once = Once::new();
static DROP: Once = Once::new();

fn run_ctors() {
    INIT.call_once(|| {
        for ctor in CTORS {
            ctor();
        }
    });
}

fn run_dtors() {
    DROP.call_once(|| {
        for dtor in DTORS {
            dtor();
        }
    });
}

#[linkme::distributed_slice(CTORS)]
fn ctor() {
    println!("Hello from linkme");
}

#[linkme::distributed_slice(DTORS)]
fn dtor() {
    println!("Goodbye from linkme");
}

struct HelloPrinter;

impl HelloPrinter {
    pub fn print(self) {
        println!("Hello from printer");
    }
}

impl Drop for HelloPrinter {
    fn drop(&mut self) {
        println!("Goodbye from printer");
    }
}

fn main() {
    run_ctors();
    println!("Hello from main");
    let printer = HelloPrinter;
    printer.print();
    run_dtors();
}
